/**
 * License Agreement.
 *
 *  JBoss RichFaces 3.0 - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.component;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * "sublist" iterator for iterate over child components instantiate from {@link UITab}
 *
 * @author shura (latest modification by $Author$)
 * @version $Revision$ $Date$
 */
class TabsIterator implements Iterator {

    static interface FilterCondition {
        public boolean isSuitable(UITab tab);
    }

    private FilterCondition filterCondition;

    /**
     *
     */
    private final UITabPanel tabsIterator;

    /**
     * Original component iterator for children
     */
    private Iterator _childs;

    /**
     * Next UITab in children.
     */
    private Object _next = null;

    public TabsIterator(UITabPanel tabPanel, FilterCondition filterCondition) {
        super();

        this.filterCondition = filterCondition;

        tabsIterator = tabPanel;
        _childs = tabsIterator.getChildren().iterator();
        _next = getNextTab();
    }

    /**
     * @param tabPanel TODO
     */
    public TabsIterator(UITabPanel tabPanel) {
        this(tabPanel, null);
    }

    /* (non-Javadoc)
      * @see java.util.Iterator#hasNext()
      */
    public boolean hasNext() {
        return _next != null;
    }

    /* (non-Javadoc)
      * @see java.util.Iterator#next()
      */
    public Object next() {
        if (null == _next) {
            throw new NoSuchElementException();
        }
        Object next = _next;
        _next = getNextTab();
        return next;
    }

    /* (non-Javadoc)
      * @see java.util.Iterator#remove()
      */
    public void remove() {
        throw new UnsupportedOperationException("tabs Iterator not support remove operation");

    }

    /**
     * Find next {@link UITab} components in children
     * @return next tab
     */
    private Object getNextTab() {
        while (_childs.hasNext()) {
            Object element = _childs.next();
            if (element instanceof UITab) {
                UITab tab = (UITab) element;
                if (filterCondition == null || filterCondition.isSuitable(tab))
                {
                    return tab;
                }
            }
        }
        return null;
    }
}