/**
 * License Agreement.
 *
 *  JBoss RichFaces 3.0 - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.renderkit.html;

import java.io.IOException;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import org.ajax4jsf.framework.renderer.AjaxRendererUtils;
import org.ajax4jsf.framework.renderer.ComponentVariables;
import org.ajax4jsf.framework.renderer.ComponentsVariableResolver;
import org.ajax4jsf.framework.renderer.HeaderResourcesRendererBase;
import org.ajax4jsf.framework.util.javascript.JSFunction;
import org.richfaces.component.UIDatascroller;
import org.richfaces.event.DataScrollerEvent;


public class DataScrollerRenderer extends HeaderResourcesRendererBase {

	protected Class getComponentClass() {
		return UIDatascroller.class;
	}

	public void encodeBegin(FacesContext context, UIComponent component)
	throws IOException {

		UIDatascroller scroller = (UIDatascroller) component;

		UIData dataTable = scroller.getDataTable();

		ComponentVariables variables = ComponentsVariableResolver.getVariables(this, component);
		variables.setVariable("dataTable", dataTable);
		if (!(scroller.getPageCount(dataTable) == 1) || (scroller.isRenderIfSinglePage()))
		{
			variables.setVariable("rendered", Boolean.TRUE);
			super.encodeBegin(context, component);
		}
	}

	public void encodeEnd(FacesContext context, UIComponent component)
	throws IOException {
		ComponentVariables variables = ComponentsVariableResolver.getVariables(this, component);

		if (Boolean.TRUE.equals(variables.getVariable("rendered")))
		{
			super.encodeEnd(context, component);
		}
	}

	public void decode(FacesContext context, UIComponent component) {

		String param = (String) getParamMap(context).get(
				component.getClientId(context));
		UIDatascroller scroller = (UIDatascroller) component;

		if (param != null) {
			new DataScrollerEvent(scroller, String.valueOf(scroller.getPageIndex()), param).queue();
		}
	}

	public ControlsState getControlsState(FacesContext context, UIDatascroller datascroller, UIData dataTable) {
		int pageIndex = datascroller.getPageIndex(dataTable);
		int pageCount = datascroller.getPageCount(dataTable);
		int minPageIdx = 1;
		int maxPageIdx = pageCount;
		int fastStep = datascroller.getFastStep();
		if (fastStep <= 1) {
			fastStep = 1;
		}

		boolean useFirst = true;
		boolean useLast = true;

		boolean useBackFast = true;
		boolean useForwFast = true;

		ControlsState controlsState = new ControlsState();
		
		if (pageIndex <= minPageIdx) {
			useFirst = false;
		}

		if (pageIndex >= maxPageIdx) {
			useLast = false;
		}

		if (pageIndex - fastStep < minPageIdx) {
			useBackFast = false;
		}

		if (pageIndex + fastStep > maxPageIdx) {
			useForwFast = false;
		}

		boolean isAuto;
		String boundaryControls = datascroller.getBoundaryControls();
		String stepControls = datascroller.getStepControls();
		String fastControls = datascroller.getFastControls();
		
		if ((isAuto = "auto".equals(boundaryControls)) || "show".equals(boundaryControls)) {
			if (isAuto) {
				controlsState.setFirstRendered(useFirst);
				controlsState.setLastRendered(useLast);
			}

			controlsState.setFirstEnabled(useFirst);
			controlsState.setLastEnabled(useLast);
		} else {
			controlsState.setFirstRendered(false);
			controlsState.setLastRendered(false);
		}

		if ((isAuto = "auto".equals(stepControls)) || "show".equals(stepControls)) {
			if (isAuto) {
				controlsState.setPreviousRendered(useFirst);
				controlsState.setNextRendered(useLast);
			}

			controlsState.setPreviousEnabled(useFirst);
			controlsState.setNextEnabled(useLast);
		} else {
			controlsState.setPreviousRendered(false);
			controlsState.setNextRendered(false);
		}

		if ((isAuto = "auto".equals(fastControls)) || "show".equals(fastControls)) {
			if (isAuto) {
				controlsState.setFastForwardRendered(useForwFast);
				controlsState.setFastRewindRendered(useBackFast);
			}

			controlsState.setFastForwardEnabled(useForwFast);
			controlsState.setFastRewindEnabled(useBackFast);
		} else {
			controlsState.setFastForwardRendered(false);
			controlsState.setFastRewindRendered(false);
		}
		
		return controlsState;
	}

	public void renderPager(FacesContext context, UIComponent component) throws IOException {
		ComponentVariables variables = ComponentsVariableResolver.getVariables(this, component);
		UIData data = (UIData) variables.getVariable("dataTable");

		ResponseWriter out = context.getResponseWriter();
		UIDatascroller scroller = (UIDatascroller) component;
		int currentPage = scroller.getPageIndex(data);

		int maxPages = scroller.getMaxPages();
		if (maxPages <= 1) {
			maxPages = 1;
		}
		int pageCount = scroller.getPageCount(data);
		if (pageCount <= 1) {
			return;
		}

		int delta = maxPages / 2;

		int pages;
		int start;
		if (pageCount > maxPages && currentPage > delta) {
			pages = maxPages;
			start = currentPage - pages / 2 - 1;
			if (start + pages > pageCount) {
				start = pageCount - pages;
			}
		} else {
			pages = pageCount < maxPages ? pageCount : maxPages;
			start = 0;
		}

		for (int i = start, size = start + pages; i < size; i++) {
			//String styleClass;
			//String style;

			boolean isCurrentPage = (i + 1 == currentPage);
			/*
               if (isCurrentPage) {
                   styleClass = scroller.getSelectedStyleClass();
                   style = scroller.getSelectedStyle();
               }
               else {
                   styleClass = scroller.getStyleClass();
                   style = scroller.getStyle();
               }

               if (style==null){
                   style="";
               }
               if (styleClass==null){
                   styleClass="";
               }
			 */

			out.startElement("td", component);
			if (isCurrentPage) {
				out.writeAttribute("align", "center", null);
				//out.writeAttribute("class", "dr-dscr-act rich-datascr-act "+scroller.getStyleClass(), null);
				out.writeAttribute("class", "dr-dscr-act rich-datascr-act", null);
				out.writeAttribute("style", "" + scroller.getSelectedStyle(), null);
			} else {
				out.writeAttribute("align", "center", null);
				//out.writeAttribute("class", "dr-dscr-inact rich-datascr-inact "+scroller.getStyleClass(), null);
				out.writeAttribute("class", "dr-dscr-inact rich-datascr-inact", null);
				//out.writeAttribute("style", ""+scroller.getStyle(), null);
				out.writeAttribute("onclick", getOnClick(context, component, Integer.toString(i + 1)), null);
			}
			out.writeText(Integer.toString(i + 1), null);
			//renderChild(context, link);
			out.endElement("td");
		}

	}

	private Map getParamMap(FacesContext context) {
		return context.getExternalContext().getRequestParameterMap();
	}

	public String getOnClick(FacesContext context, UIComponent component, String value) {

		JSFunction function = AjaxRendererUtils.buildAjaxFunction(component,
				context);
		UIDatascroller scroller = (UIDatascroller) component;
		Map eventOptions = AjaxRendererUtils.buildEventOptions(context, component);
		Map parameters = (Map) eventOptions.get("parameters");
		parameters.put(scroller.getClientId(context), value);
		function.addParameter(eventOptions);
		StringBuffer buffer = new StringBuffer();
		function.appendScript(buffer);
		buffer.append("; return false;");
		String script = buffer.toString();
		return script;
	}


	/**
	 * Creates HtmlAjaxCommandLink sets its id, value and reRender and UIParameter
	 */
	public boolean getRendersChildren() {
		return true;
	}
}
